; Instructions
; ============
;
; 1. Download and unpack NSSM
; ---------------------------
;
; https://nssm.cc/download
;
; Currently version: 2.24
; Unpack into win-installer/nssm-2.24
;
; 2. Download and unpack node LTS for windows
; -------------------------------------------
;
; https://nodejs.org/en/download/
;
; Currently version: 10.16.3 32-bit
; Unpack into win-installer/node-v10.16.3-win-x86
;
; 3. Build and pack appconnector
; ------------------------------
;
; cd ..
; yarn build
; yarn pack
;
; 4. Update version strings
; --------------------------
; 
; Update AppVersion below
; Update win-installer/package.json
;
; 5. Build and test
; -----------------
;
; Build installer using inno setup compiler 
; - It's possible to do on mac/linux if wine is installed using command: npx innosetup-compiler installer.iss
; 
; Test in windows

#define AppName "Evoko App Connector"
#define AppVersion "2.0.2"
#define AppPublisher "Evoko Unlimited AB"
#define AppURL "http://www.evoko.se"

#define ASSETS_PREFIX "..\assets\"

#define NODE_PREFIX ".\node-v10.16.3-win-x86"
#define NSSM_PREFIX ".\nssm-2.24"
#define PACKAGE_TGZ "..\evoko-getaroom-appconnector-v" + AppVersion + ".tgz"
#define PACKAGE_JSON ".\package.json"
#define RUN_BAT ".\run.bat"

#define SERVICE_ID   "evoko-app-connector"
#define SERVICE_NAME "Evoko App Connector"
#define SERVICE_DESCRIPTION "Gateway server that allows Evoko Get A Room mobile app to communicate with Evoko Home"

#define CONFIG_PATH "{commonappdata}\Evoko"

#define NODE_BIN "{app}\node\node.exe"
#define NPM_BIN "{app}\node\npm.cmd"
#define SERVER_BIN "{app}\node_modules\@evoko\getaroom-appconnector\bin\appconnector.js"
#define NSSM_BIN "{app}\nssm\nssm.exe"
#define RUN_BAT_BIN "{app}\run.bat"

[Setup]
; Do never change AppId
AppId={{DDEC3CFD-091D-432A-94DE-D52720AB0EEE}
AppName={#AppName}
AppVersion={#AppVersion}
AppVerName={#AppName} {#AppVersion}
AppPublisher={#AppPublisher}
AppPublisherURL={#AppURL}
AppSupportURL={#AppURL}
AppUpdatesURL={#AppURL}
DefaultDirName={pf}\Evoko\App Connector
DefaultGroupName={#AppName}
OutputBaseFilename=evoko-getaroom-appconnector-v{#AppVersion}-unsigned
OutputDir=.
Compression=lzma
SolidCompression=yes
WizardStyle=modern

[Dirs]
Name: "{#CONFIG_PATH}\ssl"; Permissions: users-modify;

[InstallDelete]
Type: filesandordirs; Name: "{app}\*"

[Files]
; Include run.bat
Source: "{#RUN_BAT}"; DestDir: "{app}"; AfterInstall: ConvertLineEndings;
; Include Package containing application as well as package.json to ease installation of package
Source: "{#PACKAGE_TGZ}"; DestDir: "{app}"; Flags: deleteafterinstall;
Source: "{#PACKAGE_JSON}"; DestDir: "{app}"; Flags: deleteafterinstall;
; Include NSSM to bea ble to install application as service
Source: "{#NSSM_PREFIX}\win32\nssm.exe"; DestDir: "{app}\nssm";
; Include Node to be able to run application
Source: "{#NODE_PREFIX}\node.exe"; DestDir: "{app}\node";
; Include NPM to be able to install package
Source: "{#NODE_PREFIX}\npm"; DestDir: "{app}\node"; Flags: deleteafterinstall;
Source: "{#NODE_PREFIX}\npm.cmd"; DestDir: "{app}\node"; Flags: deleteafterinstall;
Source: "{#NODE_PREFIX}\node_modules\*"; DestDir: "{app}\node\node_modules"; Flags: recursesubdirs deleteafterinstall;
Source: "{#ASSETS_PREFIX}\config.toml"; DestDir: "{#CONFIG_PATH}"; Flags: onlyifdoesntexist uninsneveruninstall; Permissions: users-modify; AfterInstall: ConvertLineEndings;

[Languages]
Name: "english"; MessagesFile: "compiler:Default.isl"

[Tasks]
Name: installservice; Description: "Install service"; 

[Run]
; Note that we need to quote all paths to handle space characters.
; See "Quoting Issues": https://nssm.cc/usage.
Filename: "{#NPM_BIN}"; Parameters: "install --production --scripts-prepend-node-path"; WorkingDir: {app}; Flags: shellexec waituntilterminated runhidden; 
Filename: "{#NSSM_BIN}"; Parameters: "stop {#SERVICE_ID}"; Flags: runhidden; Tasks: installservice;
Filename: "{#NSSM_BIN}"; Parameters: "remove {#SERVICE_ID} confirm"; Flags: runhidden; Tasks: installservice;
Filename: "{#NSSM_BIN}"; Parameters: "install {#SERVICE_ID} ""{#NODE_BIN}"" """"""{#SERVER_BIN}"""""" start"; Flags: runhidden; Tasks: installservice;
Filename: "{#NSSM_BIN}"; Parameters: "set {#SERVICE_ID} Start SERVICE_AUTO_START"; Flags: runhidden; Tasks: installservice; 
Filename: "{#NSSM_BIN}"; Parameters: "set {#SERVICE_ID} DisplayName {#SERVICE_NAME}"; Flags: runhidden; Tasks: installservice; 
Filename: "{#NSSM_BIN}"; Parameters: "set {#SERVICE_ID} Description {#SERVICE_DESCRIPTION}"; Flags: runhidden; Tasks: installservice; 
Filename: "{#NSSM_BIN}"; Parameters: "start {#SERVICE_ID}"; Flags: runhidden; Tasks: installservice;
Filename: "notepad.exe"; Parameters: """{#CONFIG_PATH}\config.toml"""; Description: "Open config file in text editor"; Flags: postinstall; AfterInstall: SetMarqueeProgress(False);

[UninstallRun]
Filename: "{#NSSM_BIN}"; Parameters: "remove {#SERVICE_ID} confirm";

[Icons]
Name: "{group}\Open config"; Filename: "notepad.exe"; Parameters: "{#CONFIG_PATH}\config.toml";
Name: "{group}\Start in foreground"; Filename: "{#RUN_BAT_BIN}"; Parameters: "start"; WorkingDir: {app};
Name: "{group}\{cm:UninstallProgram,{#AppName}}"; Filename: "{uninstallexe}"

[Code]
procedure SetMarqueeProgress(Marquee: Boolean);
begin
  if Marquee then
  begin
    WizardForm.ProgressGauge.Style := npbstMarquee;
  end
    else
  begin
    WizardForm.ProgressGauge.Style := npbstNormal;
  end;
end;

[Code]
const
    LF = #10;
    CR = #13;
    CRLF = CR + LF;

procedure ConvertLineEndings();
var
    FilePath : String;
    FileContents : String;
    AnsiFileContents : AnsiString;
begin
    FilePath := ExpandConstant(CurrentFileName);
    LoadStringFromFile(FilePath, AnsiFileContents);
    FileContents := String(AnsiFileContents);
    if Pos(CRLF, FileContents) = 0 then
    begin
        StringChangeEx(FileContents, LF, CRLF, False);
        SaveStringToFile(FilePath, AnsiString(FileContents), False);
    end;
end;
